import * as i0 from '@angular/core';
import { Injectable } from '@angular/core';
import { BehaviorSubject } from 'rxjs';
import { map } from 'rxjs/operators';

class MapState {
    constructor(zoom, center, options, extent = [-180.0, -90.0, 180.0, 90.0], time = new Date().toISOString(), viewAngle = 0, rotation = 0) {
        const defaultOptions = {
            maxzoom: 0,
            minzoom: 0,
            notifier: 'map'
        };
        this.zoom = zoom;
        this.center = center;
        this.extent = extent;
        this.time = time;
        this.viewAngle = viewAngle;
        this.rotation = rotation;
        this.options = Object.assign(defaultOptions, options);
    }
    sameCenter(center) {
        if (this.center.lat === center.lat && this.center.lon === center.lon) {
            return true;
        }
        else {
            return false;
        }
    }
    sameZoom(zoom) {
        if (this.zoom === zoom) {
            return true;
        }
        else {
            return false;
        }
    }
    sameExtent(extent) {
        const len = extent.length;
        let isSame = false;
        if (this.extent.length === len) {
            isSame = this.extent.every((v, i) => extent[i] === v);
        }
        return isSame;
    }
    sameNotifier(notifier) {
        if (this.options.notifier === notifier) {
            return true;
        }
        else {
            return false;
        }
    }
}

const initialState = new MapState(0, { lat: 0, lon: 0 });
class MapStateService {
    constructor() {
        this.mapState = new BehaviorSubject(initialState);
        this.lastAction = new BehaviorSubject(null);
    }
    getMapState() {
        return this.mapState;
    }
    setMapState(state) {
        if (!state) {
            return;
        }
        this.lastAction.next('setState');
        if (state instanceof MapState) {
            const newState = new MapState(state.zoom, state.center, state.options, state.extent, state.time, state.viewAngle, state.rotation);
            this.mapState.next(newState);
        }
        else {
            const stateOptions = { ...{ notifier: 'user' }, ...state.options };
            const newState = new MapState(state.zoom, state.center, stateOptions, state.extent, state.time, state.viewAngle, state.rotation);
            this.mapState.next(newState);
        }
    }
    getExtent() {
        return this.mapState.pipe(map((state) => state.extent));
    }
    setExtent(extent, notifier = 'user') {
        if (!Array.isArray(extent)) {
            return;
        }
        this.lastAction.next('setExtent');
        const state = this.getMapState().getValue();
        state.options.notifier = notifier;
        const newState = new MapState(state.zoom, state.center, state.options, extent, state.time, state.viewAngle, state.rotation);
        this.mapState.next(newState);
    }
    setTime(time, notifier = 'user') {
        this.lastAction.next('setTime');
        const state = this.getMapState().getValue();
        state.options.notifier = notifier;
        if (time instanceof Date) {
            state.time = time.toISOString();
        }
        else {
            state.time = time;
        }
        this.setMapState(state);
    }
    /**
     * @param angle
     * This is not available for OpenLayers (only rotation)
     *
     * For Maplibre the default is from 0 to 60. Can be increasd with setMaxPitch (0-85). Values greater than 60 degrees are experimental and may result in rendering issues.
     *
     * For Cesium values greater than 90, is like looking from the ground up to the sky. Values greater than 180 also rotate the globe.
     */
    setViewAngle(angle, notifier = 'user') {
        if (isNaN(angle)) {
            return;
        }
        this.lastAction.next('setAngle');
        const state = this.getMapState().getValue();
        state.options.notifier = notifier;
        const newState = new MapState(state.zoom, state.center, state.options, state.extent, state.time, angle, state.rotation);
        this.mapState.next(newState);
    }
    setRotation(rotation, notifier = 'user') {
        if (isNaN(rotation)) {
            return;
        }
        this.lastAction.next('setRotation');
        const state = this.getMapState().getValue();
        state.options.notifier = notifier;
        const newState = new MapState(state.zoom, state.center, state.options, state.extent, state.time, state.viewAngle, rotation);
        this.mapState.next(newState);
    }
    getLastAction() {
        return this.lastAction;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.12", ngImport: i0, type: MapStateService, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.2.12", ngImport: i0, type: MapStateService, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.12", ngImport: i0, type: MapStateService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [] });

/*
 * Public API Surface of services-map-state
 */

/**
 * Generated bundle index. Do not edit.
 */

export { MapState, MapStateService };
//# sourceMappingURL=dlr-eoc-services-map-state.mjs.map
